# Importare la libreria 'pygame'
import pygame
import pygame.gfxdraw
from random import randint
 
# Inizializzare 'pygame'
pygame.init()
 
# Definire i colori che useremo in formato RGB
GREY =  (230, 230, 230)
GREEN = (  0, 180,   0)

# Creare uno schermo largo 400 e alto 300
screen = pygame.display.set_mode( [400,300] )

# Questa procedura traccia un punto sullo schermo.
# I calcoli per x e y servono a scalare le posizioni dei punti
# in modo che si trovino correttamente sullo schermo.
def plot_point( p ):
    x = (p[0]+4.6)/10 * 400
    y = (9.5-p[1])/10 * 300
    r = pygame.draw.rect(screen,GREEN,(x,y,2,2) )
    pygame.display.update(r)

# Questa procedura traccia una chiazza circolare verde (GREEN) sullo schermo
def draw_blob( p, size ):
    x = int(p[0])
    y = int(p[1])
    pygame.gfxdraw.filled_circle(screen,x,y,size*2+2,GREEN)
    pygame.display.update()
    print( "(",x,",",y,") size ",size )
    pygame.time.wait( 20 )

# apply_transform() è una funzione che prende la posizione di un punto
# e restituisce una nuova posizione.
#   t è una delle 'trasformate affini'.
#   p è un punto a cui viene applicata la trasformata
#   Questa funzione ritorna la posizione del punto dopo
#   l'applicazione della trasformata.
def apply_transform( t, p ):
    return ( t[0]*p[0]+t[1]*p[1]+t[4],t[2]*p[0]+t[3]*p[1]+t[5])

# Chiede a pygame se l'utente ha fatto clic sulla X per chiudere.
def check_for_quit():
    for event in pygame.event.get():
        if event.type == pygame.QUIT: # Se l'utente ha fatto clic su chiudi
            pygame.quit()
            return True
    return False

# Continua a verificare gli eventi
# SE si incontra una draw_function, si ridisegna lo schermo.
def do_eventloop( draw_function ) :
    clock = pygame.time.Clock()
    while True:
        # Questo limita il ciclo while a un max di 10 volte al secondo.
        # Lasciato così com'è farà usare tutta la CPU possibile.
        clock.tick(10)
        if check_for_quit() :
            return
        if( draw_function ) :
            # Pulisce lo schermo e lo rende grigio
            screen.fill(GREY)
            draw_function()
            # Mostra quello che è stato tracciato.
            pygame.display.update()

# Chiama una sola volta una funzione di disegno, poi aspetta che l'utente
# faccia clic su X per chiudere.
def draw_once( function ):
    print("Inizio")
    # Pulisce lo schermo e lo rende grigio.
    screen.fill(GREY)
    # Mostra lo sfondo grigio.
    pygame.display.update()
    function()
    # Mostra quello che è stato tracciato.
    pygame.display.update()
    print( 'Finito')    
    # Esegue il ciclo dell'evento, ma non disegna nulla.
    draw_nothing = 0
    do_eventloop( draw_nothing )

# Esegue le stesse operazioni di do_eventloop.
# Continua a disegnare sullo schermo finché l'utente non fa clic su X
# per chiudere.
def draw_repeatedly( draw_function ):
    do_eventloop( draw_function )

#--- Fine del codice Boilerplate


    
# traccia fili d'erba.
# i è il numero del filo d'erba
# s e t sono controllate dalla posizione del mouse
def draw_blade( i, s, t ) :
    x = i * 10 + 50
    # dx sposta le punte dell'erba a destra o a sinistra.
    # randint() cambia dx di un valore casuale in modo che l'erba
    # 'si muova nel vento'.
    # 2*t*(i-15) fa allontanare le punte dell'erba.
    dx = randint( -10, 10 )/10 + 2*t*(i-15)
    shape = [(x-2,100),(x+2,100),(x+dx+s,50),(x+dx-s,50)]
    pygame.gfxdraw.aapolygon( screen, shape , GREEN)
    pygame.gfxdraw.filled_polygon( screen, shape , GREEN)
    
# itera per 30 fili d'erba
def draw_grass():
    pos = pygame.mouse.get_pos()
    for i in range(30):
        draw_blade(i, pos[0]/200, pos[1]/300 )

draw_repeatedly( draw_grass )
