# Importa la libreria 'pygame'
import pygame
import pygame.gfxdraw
from random import randint
 
# Inizializza 'pygame'
pygame.init()
 
# Definisce i colori che useremo in formato RGB
GREY =  (230, 230, 230)
GREEN = (  0, 180,   0)

# Crea uno schermo largo 400 e alto 300
screen = pygame.display.set_mode( [400,300] )

# Questa procedura traccia un punto sullo schermo.
# I calcoli per x e y servono a scalare le posizioni dei punti
# in modo che si trovino correttamente sullo schermo.
def plot_point( p ):
    x = (p[0]+4.6)/10 * 400
    y = (9.5-p[1])/10 * 300
    r = pygame.draw.rect(screen,GREEN,(x,y,2,2) )
    pygame.display.update(r)

# Questa procedura traccia una chiazza circolare verde (GREEN) sullo schermo
def draw_blob( p, size ):
    x = int(p[0])
    y = int(p[1])
    pygame.gfxdraw.filled_circle(screen,x,y,size*2+2,GREEN)
    pygame.display.update()
    print( "(",x,",",y,") size ",size )
    pygame.time.wait( 20 )

# apply_transform() è una funzione che prende la posizione di un punto
# e restituisce una nuova posizione.
#   t è una delle 'trasformate affini'.
#   p è un punto a cui viene applicata la trasformata
#   Questa funzione ritorna la posizione del punto dopo
#   l'applicazione della trasformata.
def apply_transform( t, p ):
    return ( t[0]*p[0]+t[1]*p[1]+t[4],t[2]*p[0]+t[3]*p[1]+t[5])

# Chiede a pygame se l'utente ha fatto clic sulla X per chiudere.
def check_for_quit():
    for event in pygame.event.get():
        if event.type == pygame.QUIT: # Se l'utente ha fatto clic su chiudi
            pygame.quit()
            return True
    return False

# Continua a verificare gli eventi
# SE si incontra una draw_function, si ridisegna lo schermo.
def do_eventloop( draw_function ) :
    clock = pygame.time.Clock()
    while True:
        # Questo limita il ciclo while a un max di 10 volte al secondo.
        # Lasciato così com'è farà usare tutta la CPU possibile.
        clock.tick(10)
        if check_for_quit() :
            return
        if( draw_function ) :
            # Pulisce lo schermo e lo rende grigio
            screen.fill(GREY)
            draw_function()
            # Mostra quello che è stato tracciato.
            pygame.display.update()

# Chiama una sola volta una funzione di disegno, poi aspetta che l'utente
# faccia clic su X per chiudere.
def draw_once( function ):
    print("Inizio")
    # Pulisce lo schermo e lo rende grigio.
    screen.fill(GREY)
    # Mostra lo sfondo grigio.
    pygame.display.update()
    function()
    # Mostra quello che è stato tracciato.
    pygame.display.update()
    print( 'Finito')    
    # Esegue il ciclo dell'evento, ma non disegna nulla
    draw_nothing = 0
    do_eventloop( draw_nothing )

# Esegue le stesse operazioni di do_eventloop.
# Continua a disegnare sullo schermo finché l'utente non fa clic su X
# per chiudere.
def draw_repeatedly( draw_function ):
    do_eventloop( draw_function )

#--- Fine del codice Boilerplate 


    

# Definisce quattro 'trasformazioni affini'
f1 = [0,0,0,0.22,0,0]
f2 = [0.85,0.04,-0.04,0.85,0,1.6]
f3 = [0.2,-0.26,0.23,0.22,0,1.6]
f4 = [-0.15,0.28,0.26,0.24,0,0.44]
    
# La funzione ricorsiva che richiama se stessa.
# Si richiama quattro volte
# Ciascuna di queste a sua volta si richiama quattro volte, 
def draw_fern( level, p ):
    if level < 1 :
        plot_point( p )
        return
    draw_fern( level-1, apply_transform( f1, p ))
    draw_fern( level-1, apply_transform( f2, p ))
    draw_fern( level-1, apply_transform( f3, p ))
    draw_fern( level-1, apply_transform( f4, p ))

def draw_the_fern():
    draw_fern( 9, (0,0) )    

draw_once( draw_the_fern )    
 

